﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using iTextSharp.text.pdf;
using System.IO;

namespace WatermarkTest
{
    public class PdfWaterMark
    {
        #region 私有变量
        private int _width;
        private int _height;
        private string _fontFamily;
        private int _fontSize;
        private bool _adaptable;
        private FontStyle _fontStyle;
        private bool _shadow;
        private string _backgroundImage;
        private Color _bgColor;
        private int _left;
        private string _saveImage;
        private int _top;
        private int _alpha;
        private int _red;
        private int _green;
        private int _blue;
        private long _quality;
        private bool _readOnly;
        #endregion
        #region 公共属性
        /**/
        /// <summary>
        /// 字体
        /// </summary>
        public string FontFamily
        {
            set { this._fontFamily = value; }
        }

        /**/
        /// <summary>
        /// 文字大小
        /// </summary>
        public int FontSize
        {
            set { this._fontSize = value; }
        }

        /**/
        /// <summary>
        /// 文字风格
        /// </summary>
        public FontStyle FontStyle
        {
            get { return _fontStyle; }
            set { _fontStyle = value; }
        }

        /**/
        /// <summary>
        /// 透明度0-255,255表示不透明
        /// </summary>
        public int Alpha
        {
            get { return _alpha; }
            set { _alpha = value; }
        }

        /**/
        /// <summary>
        /// 水印文字是否使用阴影
        /// </summary>
        public bool Shadow
        {
            get { return _shadow; }
            set { _shadow = value; }
        }

        public int Red
        {
            get { return _red; }
            set { _red = value; }
        }

        public int Green
        {
            get { return _green; }
            set { _green = value; }
        }

        public int Blue
        {
            get { return _blue; }
            set { _blue = value; }
        }

        /**/
        /// <summary>
        /// 底图
        /// </summary>
        public string BackgroundImage
        {
            set { this._backgroundImage = value; }
        }

        /**/
        /// <summary>
        /// 水印文字的左边距
        /// </summary>
        public int Left
        {
            set { this._left = value; }
        }


        /**/
        /// <summary>
        /// 水印文字的顶边距
        /// </summary>
        public int Top
        {
            set { this._top = value; }
        }

        /**/
        /// <summary>
        /// 水印图片保存位置
        /// </summary>
        public string SaveImage
        {
            set { this._saveImage = value; }
        }

        /**/
        /// <summary>
        /// 生成图片的宽度
        /// </summary>
        public int Width
        {
            get { return _width; }
            set { _width = value; }
        }

        /**/
        /// <summary>
        /// 生成图片的高度
        /// </summary>
        public int Height
        {
            get { return _height; }
            set { _height = value; }
        }

        /**/
        /// <summary>
        /// 若文字太大，是否根据背景图来调整文字大小，默认为适应
        /// </summary>
        public bool Adaptable
        {
            get { return _adaptable; }
            set { _adaptable = value; }
        }

        public Color BgColor
        {
            get { return _bgColor; }
            set { _bgColor = value; }
        }

        /**/
        /// <summary>
        /// 输出图片质量，质量范围0-100,类型为long
        /// </summary>
        public long Quality
        {
            get { return _quality; }
            set { _quality = value; }
        }

        public bool ReadOnly
        {
            get { return _readOnly; }
            set { _readOnly = value; }
        }

        #endregion
        public PdfWaterMark()
        {
            _width = 460;
            _height = 40;
            _fontFamily = "华文行楷";
            _fontSize = 20;
            _fontStyle = FontStyle.Regular;
            _adaptable = true;
            _shadow = false;
            _left = 0;
            _top = 0;
            _alpha = 255;
            _red = 0;
            _green = 0;
            _blue = 0;
            _backgroundImage = "";
            _quality = 100;
            _bgColor = Color.FromArgb(255, 229, 229, 229);
            _readOnly = false;
        }
        /// <summary>
        /// 为pdf加水印
        /// </summary>
        /// <param name="inputPdfFile">输入pdf文件</param>
        /// <param name="outputPdfFile">输出pdf文件</param>
        /// <param name="waterMark">水印文字</param>
        /// <returns></returns>
        public bool CreateWaterMark(string inputPdfFile, string outputPdfFile, string waterMark)
        {
            string watermarkeImg = CreateWaterMarkImage(waterMark);//生成水印图片
            if (watermarkeImg != string.Empty)
            {  
                PdfReader pdfReader = null;
                PdfStamper pdfStamper = null;
                try
                {
                    pdfReader = new PdfReader(inputPdfFile);
                    int numberOfPages = pdfReader.NumberOfPages;
                    iTextSharp.text.Rectangle psize = pdfReader.GetPageSize(1);
                    float width = psize.Width;
                    float height = psize.Height;
                    pdfStamper = new PdfStamper(pdfReader, new FileStream(outputPdfFile, FileMode.Create));
                    if (ReadOnly)
                        pdfStamper.SetEncryption(PdfWriter.STRENGTH40BITS, null, null, PdfWriter.AllowScreenReaders);


                    PdfContentByte waterMarkContent;
                    //水印图片
                    iTextSharp.text.Image imageWaterMark = iTextSharp.text.Image.GetInstance(watermarkeImg);
                    //imageWaterMark.GrayFill = 5;//透明度，灰色填充
                    imageWaterMark.Rotation = 45;//旋转
                    imageWaterMark.RotationDegrees = 45;//旋转角度
                    //每一页加水印,也可以设置某一页加水印 
                    for (int i = 1; i <= numberOfPages; i++)
                    {
                        waterMarkContent = pdfStamper.GetOverContent(i);
                        //图片水印
                        for (float h = 50; h < height; h = h + (_height+50))
                        {
                            for (float w = 0; w < width; w = w + _width)
                            {
                                imageWaterMark.SetAbsolutePosition(w, h);
                                waterMarkContent.AddImage(imageWaterMark);
                            }
                        }
                    }
                    return true;
                }
                catch
                {
                    return false;
                }
                finally
                {

                    if (pdfStamper != null)
                        pdfStamper.Close();

                    if (pdfReader != null)
                        pdfReader.Close();
                }
            }
            return false;
        }

        private string CreateWaterMarkImage(string waterMark)
        {
            try
            {
                string saveImagePath = _saveImage + DateTime.Now.ToString("yyyyMMddHHmmssfff") + ".jpg";
                Bitmap bitmap;
                Graphics g;
                //使用纯背景色
                if (this._backgroundImage.Trim() == "")
                {
                    bitmap = new Bitmap(this._width, this._height, PixelFormat.Format64bppArgb);
                    g = Graphics.FromImage(bitmap);
                    //g.Clear(this._bgColor);
                    g.Clear(Color.Transparent);
                }
                else
                {
                    bitmap = new Bitmap(Image.FromFile(this._backgroundImage));
                    g = Graphics.FromImage(bitmap);
                }
                g.SmoothingMode = SmoothingMode.HighQuality;
                g.InterpolationMode = InterpolationMode.HighQualityBicubic;
                g.CompositingQuality = CompositingQuality.HighQuality;

                Font f = new Font(_fontFamily, _fontSize, _fontStyle);
                SizeF size = g.MeasureString(waterMark, f);

                // 调整文字大小直到能适应图片尺寸
                while (_adaptable == true && size.Width > bitmap.Width)
                {
                    _fontSize--;
                    f = new Font(_fontFamily, _fontSize, _fontStyle);
                    size = g.MeasureString(waterMark, f);
                }

                Brush b = new SolidBrush(Color.FromArgb(_alpha, _red, _green, _blue));
                StringFormat StrFormat = new StringFormat();
                StrFormat.Alignment = StringAlignment.Near;

                if (this._shadow)
                {
                    Brush b2 = new SolidBrush(Color.FromArgb(90, 0, 0, 0));
                    g.DrawString(waterMark, f, b2, _left + 2, _top + 1);
                }
                g.DrawString(waterMark, f, b, new PointF(_left, _top), StrFormat);

                //bitmap.Save(saveImagePath, ImageFormat.Jpeg);
                bitmap.Save(saveImagePath, ImageFormat.Png);
                bitmap.Dispose();
                g.Dispose();
                return saveImagePath;
            }
            catch
            {
                return string.Empty;
            }
        }
    }
}
